from uuid import UUID
from core import helpers, logger, preview


class UUIDMgr:
    """This will handle both UUID and CLSIDs"""

    def __init__(self, blob: bytes, payload_preview: bool):
        self.blob = blob
        self.blob_size = len(blob)
        self.payload_name = "const char* payload[]"
        self.chunk_size = 16
        self.nop = b"\x90"
        self.payload_preview = payload_preview

    def mask(self) -> dict[str:str]:
        """Mask the data as a X"""

        # format the blob!
        return self.format_blob()

    def format_blob(self) -> str:
        """Mask the data as a list of UIDs"""

        # as the bytes are looped through, this will track how much is remaining
        remainder = self.blob_size

        # start at 0, it will increment at 16 and will indicate where to start reading the chunk from
        start = 0

        uuids: list[str] = []

        # loop over 16 bytes at a time
        for idx in range(start, self.blob_size, self.chunk_size):

            # subtract 16 from the remainder
            remainder -= self.chunk_size

            # set where to start readin the chunk from
            read_from = start

            # where to finish reading
            read_to = idx + self.chunk_size

            # increment the start for next time
            start += self.chunk_size

            # read the chunk. for example: 0-16, 16-32, etc.
            chunk = self.blob[read_from:read_to]

            # if the chunk isnt 16, pad it with 0x90
            if len(chunk) < 16:
                padding = 16 - len(chunk)
                chunk = chunk + (self.nop * padding)

            # create the uuid
            uuid = UUID(bytes_le=chunk)

            # append to the list
            uuids.append(str(uuid))

        if self.payload_preview:
            logger.info("Loading payload preview:")
            preview.print_payload_preview('\n'.join(map(str, uuids)), "c")
        
        # call the helper function with the payload declaration string and the list of code to transform
        payload: str = helpers.get_c_var(self.payload_name, uuids)

        # return!
        return payload



from core import logger
from pathlib import Path
import random, string
from core import logger


def get_bytes_from_file(fpath: str) -> bytes:
    """Read a file and return its bytes"""
    try:
        fp: Path = Path(fpath)
        return fp.read_bytes()
    except Exception as e:
        logger.bad(f"Failed to read {fpath}: {str(e)}")
        return None


def random_string(size: int) -> str:
    """Get a random string"""
    return "".join(
        random.choice(string.ascii_uppercase + string.digits) for _ in range(size)
    )


def chunker(to_chunk: list, size: int) -> list:
    """Yield successive n-sized chunks from to_chunk."""
    for i in range(0, len(to_chunk), size):
        yield to_chunk[i : i + size]


def write_text_to_file(name: str, data: str) -> None:
    """Take in the name of the mask and turn it into a .c filename"""
    try:
        fp: Path = Path(name)
        fp.write_text(data)
        logger.good(f"Written to {fp}")
    except Exception as e:
        logger.bad(f"Failed to write {name}: {str(e)}")
    # doesnt really matter if it works or not, this is the last step and the try/except will tell the user what went wrong anyway
    return


def get_c_var(declaration: str, code: list[str]) -> str:
    """Given a C Variable declaration and a list of (lines of) code, format it into something C will be happy with"""

    # this will contain the final payload
    payload: str = ""

    # add the declaration and open the braces
    payload += f"{declaration} = {{\n"

    # for each line of code...
    for idx, line in enumerate(code):

        # if its the last line, dont add a comma
        if idx == len(code) - 1:
            end = "\n"
        else:
            end = ",\n"

        # escape any "
        if '"' in line:
            line = line.replace('"', '\\"')

        # add it
        payload += f'    "{line}"{end}'

    # close stuff
    payload += "};"

    return payload


def get_size_filecontent_size(path: str) -> str:
    """Wrap the read file from disk function to return the len of the bytes"""
    return str(len(get_bytes_from_file(path)))

def get_xor_logic_c(key: int) -> str:
    """Return the XOR logic"""

    if key == 0:
        return ""
    else:
        return f"for (int i = 0; i < count; ++i)\n\t{{\n\t\tshellcode[i] = (unsigned char)shellcode[i] ^ {hex(key)};\n\t}}"
    


from core.mask.MaskFactory import get_factories
from core import logger, helpers
from pathlib import Path


class DecodeHandler:
    def __init__(self, mask: str) -> None:
        self.mask = mask
        self.path: Path = Path()
        self.c_stubs = list(self.path.glob("core/decoder/stubs/*.c"))

    def get_stub(self) -> str:
        if self.mask not in get_factories().keys():
            return False

        stub_path = None

        try:
            stub_path = [i for i in self.c_stubs if self.mask in str(i)][0]
        except IndexError:
            stubbo = self.path.glob("core/decoder/stubs/*.c")
            logger.bad(f"Failed to find a C stub for '{self.mask}' within 'core/decoder/stubs'!")
            logger.bad(f"  Identified {len(self.c_stubs)} stubs within 'core/decoder/stubs':")
            stub_path = [i for i in self.c_stubs if self.mask in str(i)]
            for stub in self.c_stubs:
                logger.bad(f"    '{stub}'")
            logger.bad("Exiting program")
            return None;

        code = stub_path.read_text()

        return code

        # outpath: Path = Path(f"{self.mask.upper()}_decoder")

        # if helpers.write_text_to_cfile(str(outpath), code):
        #     return True
        # else:
        #     return False



class SVGMgr:
    def __init__(self, blob: bytes):
        self.blob = blob
        self.payload_name = "unsigned char* payload[]"
        self.chunk_size = 10

    def mask(self) -> dict[str:str]:
        """Mask the data as a X"""

        # format the blob!
        return self.format_blob()

    def format_blob(self):
        """Below is an example of a generic c formatter"""

        # convert the blob into hex
        hex_data: list[str] = utilities.get_bytes_as_hex_list(self.blob)

        # create a c array of comma seperated hex
        payload = (
            f"{self.payload_name} = {{ {utilities.join_hex_by_comma(hex_data)} }};"
        )

        data: dict[str, str] = {"payload": payload, "name": "SVG"}

        # return it
        return data



from abc import ABC, abstractmethod
from core.mask.svg import SVGMgr
from core.mask.css import CSSMgr
from core.mask.uuid import UUIDMgr
from core.mask.csv import CSVMgr


class MaskHandler(ABC):
    """Basic representation of a mask handler."""

    def __init__(self):
        self.name = ""

    @abstractmethod
    def mask(self, blob: bytes, payload_preview: bool):
        """mask some data"""


class SVGMasker(MaskHandler):
    """mask with SVG"""

    def mask(self, blob: bytes, payload_preview: bool):
        SVG = SVGMgr(blob, payload_preview)
        return SVG.mask()

class CSSMasker(MaskHandler):
    """mask with CSS"""

    def mask(self, blob: bytes, payload_preview: bool):
        CSS = CSSMgr(blob, payload_preview)
        return CSS.mask()

class CSVMasker(MaskHandler):
    """mask with CSV"""

    def mask(self, blob: bytes, payload_preview: bool):
        CSV = CSVMgr(blob, payload_preview)
        return CSV.mask()

class UUIDMasker(MaskHandler):
    """mask with UUID"""

    def mask(self, blob: bytes, payload_preview: bool):
        UUID = UUIDMgr(blob, payload_preview)
        return UUID.mask()


class MaskFactory(ABC):
    """Factory that represents a the listener."""

    def __init__(self):
        self.name = ""

    @abstractmethod
    def get_mask_type(self) -> MaskHandler:
        """Returns a new handler for the listener"""


class SVG(MaskFactory):
    """Return the correct SVG Masker object"""

    def __init__(self):
        self.name = "SVG"

    def get_mask_type(self) -> MaskHandler:
        return SVGMasker()

class CSS(MaskFactory):
    """Return the correct CSS Masker object"""

    def __init__(self):
        self.name = "CSS"

    def get_mask_type(self) -> MaskHandler:
        return CSSMasker()

class CSV(MaskFactory):
    """Return the correct CSV Masker object"""

    def __init__(self):
        self.name = "CSV"

    def get_mask_type(self) -> MaskHandler:
        return CSVMasker()

class UUID(MaskFactory):
    """Return the correct UUID Masker object"""

    def __init__(self):
        self.name = "UUID"

    def get_mask_type(self) -> MaskHandler:
        return UUIDMasker()


class CLSID(MaskFactory):
    """Return the correct CLSID Masker object"""

    def __init__(self):
        self.name = "CLSID"

    # NOTE: This is a rare thing. A UUID is a CLSID, but the WinAPI code to parse this is different.
    # So, it will just return the UUID because there isnt any point redoing it all.
    def get_mask_type(self) -> MaskHandler:
        return UUIDMasker()

def get_factories() -> dict[str, MaskFactory]:
    return {"svg": SVG(), "css": CSS(), "csv": CSV(), "uuid": UUID(), "clsid": CLSID()}


def get_mask_factory(mask_required: str) -> MaskFactory:
    """Determine the Mask Class"""
    factories:[str, MaskFactory] = get_factories()
    if mask_required.lower() in factories:
        return factories[mask_required]



import argparse
from core import banner


class Args:
    """Class to handle the arguments"""

    def get_args():
        """Parse all the arguments!"""

        # the base argument parser
        parser = argparse.ArgumentParser(
            prog="Bluffy",
            description="",
            formatter_class=argparse.RawDescriptionHelpFormatter,
            add_help=True,
        )

        # Bin to read
        parser.add_argument(
            "-b",
            "--bin",
            help="Specify bin file to load",
            metavar=""
        ),

        # Always required, but the choices[] param is needed too
        parser.add_argument(
            "-m",
            "--mask",
            help="Specify the mask for the shellcode",
            choices=["svg", "css", "uuid", "clsid", "csv"],
            metavar="",
        )

        # Always required, but the choices[] param is needed too
        parser.add_argument(
            "-x",
            "--xor",
            help="XOR payload",
            action="store_true"
        )

        # Preview the created file after ... creation
        parser.add_argument(
            "-p",
            "--preview",
            help="Preview after creation",
            action="store_true"
        )

        # Preview the created file after ... creation
        parser.add_argument(
            "-pp",
            "--payload-preview",
            help="Preview just payload after creation",
            action="store_true"
        )


        # Simple, list the available masks
        parser.add_argument(
            "--list",
            help="List all the available masks",
            action="store_true"
        )

        # parse all the flags
        args = parser.parse_args()

        # handle what happens if nothing is passed
        if not any(vars(args).values()):
            # if nothing is passed, print the help
            parser.print_help()
            quit()
        else:
            return args



from dataclasses import dataclass


@dataclass(order=True)
class Templater:
    """Dataclass to hold all the template variable names used throughout bluffy"""

    # payload
    payload: str = "###PAYLOAD###"

    # Size of the payload
    size: str = "###SIZE###"

    # Logic to handle de-XOR-ing (aka XOR)
    xor_logic: str = "###XOR_LOGIC###"


import random
from core import helpers, logger, preview


class SVGMgr:
    def __init__(self, blob: bytes, payload_preview: bool):
        self.blob = blob
        self.blob_size = len(blob)
        self.payload_name = "const char* payload[]"
        self.chunk_size = 16
        self.nop = b"\x90"
        self.payload_preview = payload_preview

    def mask(self) -> None:
        """Mask the data as a X"""

        # format the blob!
        return self.format_blob()

    def format_blob(self):
        """format the blob into an svg!"""

        # as the bytes are looped through, this will track how much is remaining
        remainder = self.blob_size

        # start at 0, it will increment at 16 and will indicate where to start reading the chunk from
        start = 0

        code: list[str] = []

        # xml declaration string
        code.append('<?xml version="1.0" standalone="no"?>')

        # open the svg tag
        code.append(
            '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="1250cm" height="1250cm">'
        )

        # add a random description
        code.append(f"<desc>{helpers.random_string(size=36)}</desc>")

        # loop over 16 bytes at a time
        for idx in range(start, self.blob_size, self.chunk_size):

            # subtract 16 from the remainder
            remainder -= self.chunk_size

            # set where to start readin the chunk from
            read_from = start

            # where to finish reading
            read_to = idx + self.chunk_size

            # increment the start for next time
            start += self.chunk_size

            # read the chunk. for example: 0-16, 16-32, etc.
            chunk = self.blob[read_from:read_to]

            # if the chunk isnt 16, pad it with 0x90
            if len(chunk) < self.chunk_size:
                padding = self.chunk_size - len(chunk)
                chunk = chunk + (self.nop * padding)

            # for every group formatted, add it into the code
            for group in self.get_svg_group(chunk):
                code.append(group)

        # close the tag
        code.append("</svg>")

        if self.payload_preview:
            logger.info("Loading payload preview:")
            preview.print_payload_preview('\n'.join(map(str, code)), "c")
        
        payload: str = helpers.get_c_var(self.payload_name, code)

        return payload

    def get_svg_group(self, chunk) -> str:
        """Create the svg <g id=x> struct"""

        code: list[str] = []

        # get a random string for the group name
        group_name: str = helpers.random_string(size=12)

        # pick a random colour
        colour: str = self.random_colour()

        # add the group declaration
        code.append(f'<g id="{group_name}" fill="{colour}">')

        # split the chunk up into a list of ints
        data: list[int] = [idx for idx in chunk]

        # amount of svg attrs that are going to be used
        attrs: int = 4

        # split all the chunks to fit into the attrs require
        for chunk_set in list(helpers.chunker(to_chunk=data, size=attrs)):
            # for every attribute
            for attr in range(0, attrs):
                # for each chunk of a chunk
                code.append(
                    f'<rect x="{chunk_set[attr]}cm" y="{chunk_set[attr]}cm" width="{chunk_set[attr]}cm" height="{chunk_set[attr]}cm"/>'
                )

        # close the group
        code.append("</g>")

        return code

    def random_colour(self):
        """Choose a random colour"""
        return random.choice(["red", "blue", "green", "purple", "orange"])



from rich.console import Console
from rich import print

console = Console()


def good(string):
    print(f"[bold green]\\_[/bold green] {string}")


def info(string):
    print(f"[bold blue]\\_[/bold blue] {string}")


def bad(string):
    print(f"[bold red]\\_[/bold red] {string}")


def masks(names: list[str]) -> None:
    print(f"[bold yellow]\\_[/bold yellow] Available masks:")
    for idx in names:
        print(f"  [bold yellow]\\_[/bold yellow] {idx}")



from core.ArgumentHandler import Args
from core import helpers
from core.mask.MaskFactory import get_mask_factory, get_factories, MaskFactory
from core.decoder.DecodeHandler import DecodeHandler
from core import logger, banner, preview
from core.Templater import Templater
import random


def get_masked_bin(args, key: int) -> str:
    """Given an input, output, and mask type: read the bytes, identify the factory, mask the bytes, write them to disk."""

    if args.bin == None or args.mask == None:
        logger.bad("Please specify -b AND -m (bin file and mask)")
        return None

    # get the bytes of the input bin
    blob: bytes = helpers.get_bytes_from_file(args.bin)

    # if that isnt possible, return.
    if blob == None:
        return None

    logger.info(f"Loaded {args.bin} ({len(blob)} bytes)")

    # get the correct factory
    factory = get_mask_factory(args.mask)

    # if that fails, return.
    if factory == None:
        return None

    # if the factory is obtained, grab the class for the mask
    mask = factory.get_mask_type()
    logger.info(f"Masking shellcode with: {factory.name}")

    # XOR
    if (key != 0):
        # python 3 should ~~~ theoretically ~~~ handle a list of integers by auto converting to bytes blob
        blob: bytes = bytes([x ^ key for x in blob])

    # give the blob to the class and perform whatever transformations... This should then return a multiline string containing the transformed data
    return mask.mask(blob, args.payload_preview)


def build_header_file(args) -> str:
    """Mask the bin and fill out the template!"""

    # 0 if XOR is not required; otherwise, generate a random integer between 1 and 254 to use as the key
    # this is done here to simplify passing through the key; neatening is encouraged, please help
    key: int = 0

    if args.xor:
        if "uuid" in args.mask.lower():
            logger.bad("Cannot use XOR and UUID. If you want things to change, create a pull request")
            logger.info("Building payload without XOR")
        else:
            key = random.randint(1,254)

    # mask the bin
    payload: str = get_masked_bin(args, key)
    if payload == None:
        return None

    # the decodehandler basically pulls the C code that is used to unmask/decode/whatever the bin
    decoder: DecodeHandler = DecodeHandler(args.mask)

    # this is the code with the tempates
    decode_stub = decoder.get_stub()

    if decode_stub == None:
        return None

    # instantiate the templater
    templater = Templater()

    # replace the payload
    decode_stub = decode_stub.replace(templater.payload, payload)

    decode_stub = decode_stub.replace(templater.xor_logic, helpers.get_xor_logic_c(key))

    # replace the size
    decode_stub = decode_stub.replace(
        templater.size, (helpers.get_size_filecontent_size(args.bin))
    )

    # return the code
    return decode_stub


def show_masks() -> None:
    """Call the get_factories() function to get the available options and pretty print them in logger.py"""
    logger.masks(get_factories())
    return


def main() -> None:
    """Get args and execute"""

    banner.show()

    # get args
    args = Args.get_args()

    # if listing, show and quit
    if args.list:
        show_masks()
        quit()

    # otherwise do the thing!
    code: str = build_header_file(args)
    if code == None:
        return

    helpers.write_text_to_file(f"{args.mask}.h", code)

    if args.preview:
        logger.info("Loading preview:")
        preview.print_preview(code, "c")

    return


if __name__ == "__main__":
    main()



from rich.console import Console
from rich.syntax import Syntax

def print_preview(code: str, language: str) -> None:
    """Pretty prints the code generated by bluffy"""
    print()
    console = Console()
    # Comment from docs:
    # “ansi_dark” or “ansi_light” which will use the color theme configured by the terminal.
    console.print(Syntax(code, language, line_numbers=True, theme="ansi_dark"))
    print()

def print_payload_preview(code: str, language: str) -> None:
    """Pretty prints the code generated by bluffy"""
    print()
    console = Console()
    # Comment from docs:
    # “ansi_dark” or “ansi_light” which will use the color theme configured by the terminal.
    console.print(Syntax(code, language, line_numbers=True, theme="ansi_dark"))
    print()